<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Model\Rule\Condition\Customer;

use Magento\SalesRule\Api\CouponRepositoryInterface;
use Magento\SalesRule\Model\Coupon;
use Magento\Framework\App\RequestInterface;
use Magento\Checkout\Model\Session;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Customer\Model\ResourceModel\CustomerRepository;

/**
 * @class Birthday
 * @package FiloBlu\RuleDayFromBirthday\Model\Rule\Condition
 */
class Birthday extends \Magento\Rule\Model\Condition\AbstractCondition
{

    const VALIDATION_MODE = 'filoblu_ruledayfrombirthday_section/birthday_coupon_id/validation_selector';
    const EXPIRATION_VALUE = 'filoblu_ruledayfrombirthday_section/birthday_coupon_id/validation_range_text';
    protected $couponRepository;
    protected $coupon;
    protected $request;
    protected $session;
    protected $manager;
    protected $scopeConfig;
    protected $storeManager;
    protected $customerSession;
    protected $customerRepository;

    /**
     * Birthday constructor.
     * @param \Magento\Rule\Model\Condition\Context $context
     * @param CouponRepositoryInterface $couponRepository
     * @param Coupon $coupon
     * @param RequestInterface $request
     * @param Session $session
     * @param ManagerInterface $manager
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface $scopeConfig
     * @param CustomerSession $customerSession
     * @param CustomerRepository $customerRepository
     * @param array $data
     */
    public function __construct(
        \Magento\Rule\Model\Condition\Context $context,
        CouponRepositoryInterface             $couponRepository,
        Coupon                                $coupon,
        RequestInterface                      $request,
        Session                               $session,
        ManagerInterface                      $manager,
        StoreManagerInterface                 $storeManager,
        ScopeConfigInterface                  $scopeConfig,
        CustomerSession                       $customerSession,
        CustomerRepository                    $customerRepository,
        array                                 $data = []
    )
    {
        parent::__construct($context, $data);
        $this->couponRepository = $couponRepository;
        $this->coupon = $coupon;
        $this->request = $request;
        $this->session = $session;
        $this->manager = $manager;
        $this->storeManager = $storeManager;
        $this->scopeConfig = $scopeConfig;
        $this->customerSession = $customerSession;
        $this->customerRepository = $customerRepository;
    }

    /**
     * Load attribute options
     *
     * @return $this
     */
    public function loadAttributeOptions()
    {
        $attributes = [
            'customer_birthday' => __("Customer Birthday")
        ];
        $this->setAttributeOption($attributes);
        return $this;
    }

    /**
     * Get input type
     * Possible values are: string, numeric, date, select, multiselect, grid, boolean
     *
     * @return string|void
     */
    public function getInputType()
    {
        return 'boolean';
    }

    /**
     * Get value element type
     *
     * We have the value element types as select, text
     * @return string
     */
    public function getValueElementType()
    {
        return 'select';
    }

    /**
     * Get value select options
     *
     * @return array|mixed
     */
    public function getValueSelectOptions()
    {
        if (!$this->hasData('value_select_options')) {
            $options = ["True" => "True"];

            $this->setData('value_select_options', $options);
        }
        return $this->getData('value_select_options');
    }

    /**
     * @param int $customerId
     * @return string|null
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCustomerDobById(int $customerId)
    {
        return $this->customerRepository->getById($customerId)->getDob();
    }

    /**
     * @param AbstractModel $model
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function validate(\Magento\Framework\Model\AbstractModel $model)
    {
        $currentStoreId = $this->storeManager->getStore()->getId() ?? 0;

        $validationMode = $this->scopeConfig->getValue(self::VALIDATION_MODE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $coupon_code = $this->session->getQuote()->getCouponCode();
        try {
            if ($validationMode == 'dob') {
                $dob = $this->getCustomerDobById($this->customerSession->getCustomerId()) ?? '';
                if ($dob == '') {
                    $val = "False";
                } else {
                    $expiration = $this->scopeConfig->getValue(self::EXPIRATION_VALUE, \Magento\Store\Model\ScopeInterface::SCOPE_STORE) ?? 20;
                    $daysFromDob = $this->getDaysFromBirthday($dob);
                    if ($daysFromDob <= $expiration && $daysFromDob >= 0) {
                        $val = "True";
                    } else {
                        $val = "False";
                    }
                }
            } else {
                $coupon_id = $this->coupon->loadByCode($coupon_code)->getCouponId();
                $validFrom = $this->couponRepository->getById($coupon_id)->getCreatedAt();
                if (strtotime($validFrom) <= strtotime(date('Y-m-d'))) {
                    $val = "True";
                } else {
                    $val = "False";
                }
            }
            $model->setData('customer_birthday', $val);

            return parent::validate($model);
        } catch (\Exception $e) {
            $this->manager->addErrorMessage($e);
        }
    }

    /**
     * @param $dateString
     * @return string
     */
    protected function updateDate($dateString)
    {
        $date_arr = date_parse($dateString);
        $date_arr['year'] = date('Y');

        return $date_arr['year'] . "-" . $date_arr['month'] . "-" . $date_arr['day'];
    }

    /**
     * @param $dob
     * @return int
     */
    protected function getDaysFromBirthday($dob)
    {
        $new_dob = $this->updateDate($dob);
        $interval = strtotime("now") - strtotime($new_dob);
        $num_of_days = (int)($interval / 86400);

        return $num_of_days;
    }
}
